<?php

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Alior Bank Raty Gateway.
 *
 * Provides a Alior Bank Raty Payment Gateway.
 *
 * @class        WC_Gateway_Alior
 * @extends        WC_Payment_Gateway
 * @version        2.2.2
 * @package        WooCommerce/Classes/Payment
 * @author        WooThemes
 */
class WC_Gateway_Alior extends WC_Payment_Gateway
{
    /**
     * @var string
     */
    private $instructions;
    /**
     * @var string
     */
    private $alior_min_order;
    /**
     * @var string
     */
    private $alior_salt;
    /**
     * @var string
     */
    private $alior_url;
    /**
     * @var string
     */
    private $alior_mcc;
    /**
     * @var string
     */
    private $alior_promocja;
    /**
     * @var string
     */
    private $alior_partner_id;
    /**
     * @var string
     */
    private $alior_subpartner_id;
    /**
     * @var void
     */
    private $alior_categories_map;
    /**
     * @var int|string
     */
    private $total;

    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        global $woocommerce;
        $this->setup_properties();
        $this->init_form_fields();
        $this->init_settings();
        $this->total = (!empty($woocommerce->session->cart_totals)) ? number_format($woocommerce->session->cart_totals['total'], 2, '.', '') : 0;
        $this->alior_promocja = $this->get_option('alior_promocja');
        $this->alior_promocja = str_replace('${sep}', '%2C', $this->alior_promocja);
        $this->alior_partner_id = $this->get_option('alior_partner_id');
        $calcUrl = 'https://kalkulator.raty.aliorbank.pl/init?supervisor='.$this->alior_partner_id.'&promotionList='.$this->alior_promocja.'&amount=';
        $this->title = $this->get_option('title');
        if ($this->get_option('alior_partner_id')) {
            $this->description = '<div style="text-align: right"><a href="' . $calcUrl . $this->total . '" target="_blank"><img src="' . plugins_url('img/alior-kalkulator-guzik.gif', __FILE__) . '" /></a></div>';
        } else {
            $this->description = '<div style="text-align: right"></div>';
        }
        $this->instructions = $this->get_option('instructions');
        $this->alior_min_order = $this->get_option('alior_min_order');
        $this->alior_salt = $this->get_option('alior_salt');
        $this->alior_url = $this->get_option('alior_url');
        $this->alior_mcc = $this->get_option('alior_mcc');
        
        $this->alior_subpartner_id = $this->get_option('alior_subpartner_id');

        $this->alior_categories_map = get_option(
            'woocommerce_alior_category'
        );

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'save_category'));
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(
            $this,
            'process_admin_options'
        ));

        add_action('woocommerce_before_thankyou', array($this, 'alior_page'));
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
    }

    public function save_category()
    {
        $categories = [];
        if (isset($_POST['alior_category'])) {
            $categories = wc_clean(wp_unslash($_POST['alior_category']));
        }

        update_option('woocommerce_alior_category', $categories);
    }

    /**
     * Setup general properties for the gateway.
     */
    protected function setup_properties()
    {
        $this->id = 'alior';
        $this->icon = apply_filters('woocommerce_alior_icon', plugins_url('img/alior.jpg', __FILE__));
        $this->method_title = __('Alior Bank Raty', 'woocommerce');
        $this->method_description = __('Have your customers pay with Raty Alior Bank.', 'woocommerce');
        $this->has_fields = false;
    }

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce'),
                'label' => __('Enable Alior Raty', 'woocommerce'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no',
            ),
            'title' => array(
                'title' => __('Title', 'woocommerce'),
                'type' => 'text',
                'description' => __('Payment method description that the customer will see on your checkout.', 'woocommerce'),
                'default' => __('Alior Bank Raty', 'woocommerce'),
                'desc_tip' => true,
            ),
            'instructions' => array(
                'title' => __('Instructions', 'woocommerce'),
                'type' => 'textarea',
                'description' => __('Installment payment', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_url' => array(
                'title' => __('Url to Alior Bank', 'alior'),
                'type' => 'text',
                'description' => __('Url address for access to installment payments', 'woocommerce'),
                'default' => __('https://raty.aliorbank.pl/directcreditsystem-frontend-consumerfinance-internet-standard/paymentprovider/', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_partner_id' => array(
                'title' => __('Partner ID', 'alior'),
                'type' => 'text',
                'description' => __('ID Partnera', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_subpartner_id' => array(
                'title' => __('SubPartner ID', 'alior'),
                'type' => 'text',
                'description' => __('ID SubPartnera', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_mcc' => array(
                'title' => __('MCC', 'alior'),
                'type' => 'text',
                'description' => __('MCC code', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_promocja' => array(
                'title' => __('Promotion', 'alior'),
                'type' => 'text',
                'description' => __('Promotion code', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_min_order' => array(
                'title' => __('Minimum order amount', 'alior'),
                'type' => 'text',
                'description' => __('The minimum order amount', 'woocommerce'),
                'default' => __('300', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_salt' => array(
                'title' => __('SALT code', 'alior'),
                'type' => 'text',
                'description' => __('SALT code', 'woocommerce'),
                'desc_tip' => true,
            ),
            'alior_category' => array(
                'type' => 'alior_category',
            ),
        );
    }

    /**
     * Generate account details html.
     *
     * @return string
     */
    public function generate_alior_category_html(): string
    {

        ob_start();
        $args = array(
            'taxonomy' => 'product_cat',
            'orderby' => 'name',
            'hierarchical' => 0,
            'hide_empty' => 0,
            'exclude' => 1
        );

        $product_categories = get_categories(
            $args
        );

        ?>
        <tr valign="top">
            <th scope="row" class="titledesc"><?php esc_html_e('Alior categories map:', 'alior'); ?></th>
            <td class="forminp" id="alior_categories">
                <div class="wc_input_table_wrapper">
                    <table class="widefat wc_input_table sortable" cellspacing="0">
                        <thead>
                        <tr>
                            <th><?php esc_html_e('Woocommerce category', 'alior'); ?></th>
                            <th><?php esc_html_e('Alior category', 'alior'); ?></th>
                        </tr>
                        </thead>
                        <tbody class="categories">
                        <?php
                        if ($product_categories) {
                            foreach ($product_categories as $cat) {
                                echo '<tr class="map">
										<td style="padding: 2px 5px">' . $cat->name . '</td>
										<td>';
                                echo '<select class="select2-choices" name="alior_category[' . $cat->term_id . ']" >' . $this->getAliorCategoryOptions($this->alior_categories_map[$cat->term_id]) . '</select></td>';

                                echo '</tr > ';
                            }
                        }
                        ?>
                        </tbody>
                    </table>
                </div>
            </td>
        </tr>
        <?php
        return ob_get_clean();

    }

    /**
     * Check If The Gateway Is Available For Use.
     *
     * @return bool
     */
    public function is_available(): bool
    {
        global $woocommerce;

        if( !is_admin() ) {
            $check_method = false;
            $amount = (float)$woocommerce->session->cart_totals['total'];
            $min_order = (float)$this->get_option('alior_min_order');

            if ($amount >= $min_order) {
                $check_method = true;
            }

            if (!$check_method) {
                return false;
            }
        }

        return parent::is_available();
    }

    /**
     * Process the payment and return the result.
     *
     * @param int $order_id
     *
     * @return array
     */
    public function process_payment($order_id): array
    {
        global $woocommerce;
        $order = wc_get_order($order_id);
        $order_number = $order->get_id();

        if ($order->get_total() > 0) {
            // Mark as on-hold (we're awaiting the payment).
            $order->update_status(
                apply_filters('woocommerce_alior_process_payment_order_status', 'on-hold', $order),
                __('Awaiting Alior Bank decision', 'woocommerce')
            );
        } else {
            $order->payment_complete();
        }

        $woocommerce->session->set('alior_order_number', $order_number);
        wc_reduce_stock_levels($order_id);
        WC()->cart->empty_cart();

        return array(
            'result' => 'success',
            'redirect' => $this->get_return_url($order),
        );
    }

    /**
     * Output for the order received page.
     */
    public function alior_page(): void
    {
        global $woocommerce;

        $order_number = $woocommerce->session->get('alior_order_number');

        if ( !empty($order_number) ) {
            $order = wc_get_order($order_number);
        }

        if ( isset($order) && is_bool($order) && empty($order_number) || empty($order) ) {
            wc_print_notice( __( 'Raty Alior: wystąpił błąd, skontaktuj się z administratorem.', 'alior' ), 'error' );
        } else {
            $firstname = $order->get_billing_first_name();
            $lastname = $order->get_billing_last_name();
            $total = number_format($order->get_total(), 2, '.', '');
            $date = date('Y-m-d');
            $time = date('H:i:s');
            $dateAndTime = $date . 'T' . $time;

            wc_get_logger()->info(print_r($this->getArticlesList($order), true));
    
            $articlesList = base64_encode($this->getArticlesList($order));
            $verificationCode = hash('sha256', $this->alior_salt . $total . $order_number . $this->alior_partner_id . $this->alior_subpartner_id . $dateAndTime . $this->alior_mcc . $firstname . $lastname . $this->get_option('alior_promocja'));
    
            $data = [
                'firstName' => $firstname,
                'lastName' => $lastname,
                'partnerId' => $this->alior_partner_id,
                'subPartnerId' => $this->alior_subpartner_id,
                'mcc' => $this->alior_mcc,
                'promotion' => $this->get_option('alior_promocja'),
                'verificationCode' => $verificationCode,
                'transactionCode' => $order_number,
                'dateAndTime' => $dateAndTime,
                'amount' => $total,
                'articlesList' => $articlesList
            ];
    
            echo wpautop(wptexturize($this->prepareSendForm($this->alior_url, $data)));
        }
    }

    /**
     * @param $url
     * @param $data
     *
     * @return string
     */
    private function prepareSendForm($url, $data): string
    {
        $result = '<form name="fr" action="' . $url . '" method="POST">';
        foreach ($data as $key => $row) {
            $result .= '<input type="hidden" name="' . $key . '" value="' . $row . '"/>';
        }
        $result .= '</form>';
        $result .= '<script type="text/javascript">document.fr.submit();</script>';

        return $result;
    }

    /**
     * Add content to the WC emails.
     *
     * @access public
     *
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
    public function email_instructions(
        $order, $sent_to_admin, $plain_text = false
    ): void
    {
        if ($this->instructions && !$sent_to_admin && $this->id === $order->get_payment_method()) {
            echo wpautop(wptexturize($this->instructions)) . PHP_EOL;
        }
    }

    /**
     * @param $order
     * @return false|string
     */
    private function getArticlesList($order)
    {
        $data = [];
        $amount = 0;
        $alior_categories_map = get_option('woocommerce_alior_category');
        foreach ($order->get_items() as $item_id => $item) {
            $catsId = wc_get_product_cat_ids($item->get_product_id());
            $categories = [];
            foreach ($catsId as $catId) {
                $categories[$catId] = $alior_categories_map[$catId];
            }
            $categories = array_unique($categories);
            $row['category'] = array_shift($categories);
            $row['name'] = substr($item->get_name(), 0, 150);
            $row['number'] = $item->get_quantity();
            $row['price'] = round($item->get_product()->get_price_including_tax(),2);
            $amount += $item->get_product()->get_price_including_tax() * $item->get_quantity();
            $data['articlesList'][] = $row;
        }

        $total = $order->get_total();
        $shipping = round((float)$order->get_shipping_total() + $order->get_shipping_tax(), 2);
        $discount = $total - ($amount + $shipping);

        if ($discount > 0){
            $data['articlesList'][] = [
                'category' => 'TKC_RABAT',
                'name' => 'RABAT',
                'number' => 1,
                'price' => round($discount, 2)
            ];
        }
        if ($shipping > 0){
            $data['articlesList'][] = [
                'category' => 'TKC_USLUGI',
                'name' => $order->get_shipping_method(),
                'number' => 1,
                'price' => $shipping,
            ];
        }
        
        return json_encode($data, true, 512);
    }

    /**
     * @param string|null $selected
     * @return string
     */
    private function getAliorCategoryOptions(?string $selected): string
    {
        $result = '<option>' . __('Choose category', 'alior') . '</option>';

        foreach ($this->getAliorCategories() as $category => $name) {
            if ($selected === $category) {
                $result .= '<option value="' . $category . '" selected="true">' . $name . '</option>';
            } else {
                $result .= '<option value="' . $category . '">' . $name . '</option>';
            }
        }

        return $result;
    }

    /**
     * @return array
     */
    private function getAliorCategories(): array
    {
        return [
            'TKC_AGDD' => 'AGD duże',
            'TKC_AGDM' => 'AGD małe',
            'TKC_AKCAUTO' => 'Akcesoria i wyposażenie pojazdu',
            'TKC_BIZUT' => 'Biżuteria, zegarki',
            'TKC_DZIECIE' => 'Wózki dziecięce, foteliki samochodowe i inne',
            'TKC_EDU' => 'Książki, materiały edukacyjne',
            'TKC_FOTO' => 'Foto i kamery',
            'TKC_INSTRMUZ' => 'Instrumenty muzyczne',
            'TKC_KOMP' => 'Komputery, gry i konsole',
            'TKC_MATBUD' => 'Materiały budowlane',
            'TKC_MEBLE' => 'Meble i wyposażenie wnętrz',
            'TKC_MOTO' => 'Motory ,motorowery, skutery - do 50 cm3',
            'TKC_NARZ' => 'Narzędzia i urządzenia',
            'TKC_ODZIEZ' => 'Odzież',
            'TKC_OGRO' => 'Nasiona, nawozy, sadzonki',
            'TKC_OGRZEW' => 'Kotły CO, JUNKERSY',
            'TKC_OKDZW' => 'Okna i drzwi',
            'TKC_OPAL' => 'Opał i węgiel',
            'TKC_OPROGR' => 'Oprogramowanie i inne komponenty komputerowe',
            'TKC_PAKIE' => 'Bony upominkowe, pakiety(inne niż medyczne)',
            'TKC_RTV' => 'RTV',
            'TKC_SPORTREH' => 'Artykuły sportowe/ rehabilitacyjne/ turystyczne',
            'TKC_SPRZKOM' => 'Monitory, drukarki i skanery',
            'TKC_SYSTEKO' => 'Systemy ekologiczne',
            'TKC_SZTUK' => 'Dzieła sztuki, antyki, dewocjonalia',
            'TKC_TEKSTYL' => 'Tekstylia domowe, pościel i koce',
            'TKC_TELE' => 'Telefony i GPS',
            'TKC_UBEZ' => 'Usługi ubezpieczeniowe',
            'TKC_USLUGI' => 'Usługi',
            'TKC_RABAT' => 'Rabat'
        ];
    }
}