<?php

/**
 * Plugin Name: Alior Bank Raty Payment Gateway
 * Plugin URI: https://aliorbank.pl
 * Description: Alior Bank Raty payment gateway for WooCommerce
 * Version: 2.2.3
 * Author: Alior Bank SA
 * Author URI: http://www.aliorbank.pl
 * License: LGPL 3.0
 * Text Domain: alior
 * Domain Path: /lang
 */
add_action('plugins_loaded', 'woocommerce_alior_init', 0);

function woocommerce_alior_init() {
    if (!class_exists('WC_Payment_Gateway')) {
        return;
    }

    load_plugin_textdomain('alior', false, dirname(plugin_basename(__FILE__)) . '/lang/');

    include_once('includes/class-wc-gateway-alior.php');

    add_filter('woocommerce_payment_gateways', 'woocommerce_alior_add_gateway');

    add_action('woocommerce_single_product_summary', 'woocommerce_alior_extend_short_description', 40);

    add_action('woocommerce_cart_collaterals', 'woocommerce_alior_extend_cart', 10);
    
    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'woocommerce_alior_action_links');
    
}

function woocommerce_alior_add_gateway($methods) {
    $methods[] = 'WC_Gateway_Alior';

    return $methods;
}

function woocommerce_alior_extend_short_description() {
    $wc = new WC_Gateway_Alior();
    $wc->init_settings();

    $promotion = $wc->get_option('alior_promocja');
    $promotion = str_replace('${sep}', '%2C', $promotion);

    $link = 'https://kalkulator.raty.aliorbank.pl/init?supervisor='.$wc->get_option('alior_partner_id').'&promotionList='. $promotion .'&amount=';
    $min_price = number_format($wc->get_option('alior_min_order'), 2, '.', '');
    $product = new WC_Product(get_the_ID());
    $price = number_format(wc_get_price_including_tax($product), 2, '.', '');
    if ($price >= $min_price) {
        echo '<a href="' . $link . $price . '" target="_blank"><img src="' . plugins_url('includes/img/alior-kalkulator-guzik.gif', __FILE__) . '" /></a>';
    }
}

function woocommerce_alior_extend_cart() {
    $wc = new WC_Gateway_Alior();
    $wc->init_settings();

    $promotion = $wc->get_option('alior_promocja');
    $promotion = str_replace('${sep}', '%2C', $promotion);

    $link = 'https://kalkulator.raty.aliorbank.pl/init?supervisor='.$wc->get_option('alior_partner_id').'&promotionList='. $promotion .'&amount=';
    $min_price = number_format($wc->get_option('alior_min_order'), 2, '.', '');
    $price =  strip_tags(WC()->cart->total);
    $price = (float) preg_replace( '#[^\d.]#', '', $price);
    if ($price >= $min_price) {
        echo '<a href="' . $link . $price . '" target="_blank"><img src="' . plugins_url('includes/img/alior-kalkulator-guzik.gif', __FILE__) . '" /></a>';
    }
}

function woocommerce_alior_action_links($links) {
    $plugin_links = array(
        '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=alior') . '">' . __('Settings', 'alior') . '</a>',
    );

    return array_merge($plugin_links, $links);
}
